var Clock = function(model, view){
    var _this = this;
    var data = model.loadData();
    var autoTab = true;
    var wasPreviouslyCounting = undefined;
    view.localizeStrings(data);
    _this.selectedHand = undefined;
    var timer;

    this.init = function() {
        data.date = new Date(data.date);
        if(data.loaded){
            if(data.clock_option) {
                _this.displayOption(data.clock_option);
            }
            if(data.tfhours) {
                _this.tfhoursToggle();
            }
            if(data.hide_seconds) {
                _this.hideSecondsToggle();
            }
            if(_this.getKeepCounting()) {
                var currentTime = _this.getSavedCurrentTime();
                var timeDifference = Math.abs((new Date()).getTime() - currentTime);
                data.date = new Date(data.date.getTime() + timeDifference);
                start();
            } else {
                set();
            }
            if(data.isSettingsOpen) {
                view.openSettings(data.getFirstTimeHelperTextTriggered, true);
            }
        } else {
            var hostObj = NB.getHostObject();
            hostObj.height =  hostObj.width;
        }
        _this.updateArrows();
        data.loaded = false;
        _this.resizeHandler();
        view.setRtl();//checks if language is Right to Left and sets UI to RTL mode if it is
        NB.getHostObject().setWidgetMinSize(180, 1);
        _this.selectionListener = new SelectionListener(_this.onSelectionChange);
        _this.addEvents();
        timer = setInterval(_this.count_time, 1000);
        setTimeout(_this.resizeHandler, 1000);
    };
    this.setKeepCounting = function(val){
        data.keep_counting = val;
        return data.keep_counting;
    };
    this.getKeepCounting = function(){
        return data.keep_counting;
    };
    this.getSavedCurrentTime= function(){
        return Date.parse(data.currentTime);
    };
    this.getDate = function() {
        return data.date;
    };
    this.setWindowSize = function(w, h) {
        var host = NB.getHostObject();
        var pad = host.width - $(window).width();
        NB.undomanager.setAutoRecord(false);
        NB.batch(function() {
            host.width = w + pad;
            host.height = h + pad;
        });
        NB.undomanager.setAutoRecord(true);
        _this.updateThumbnail();
    };
    this.setAnnotationSize = function(w, h) {
        var host = NB.getHostObject();
        NB.undomanager.setAutoRecord(false);
        NB.batch(function() {
            host.width = w;
            host.height = h;
        });
        NB.undomanager.setAutoRecord(true);
        _this.updateThumbnail();
    };
    this.resizeHandler = function(){
        view.layoutBig(data.isSettingsOpen);
        view.layoutAnalogClock(data.clock_option, data.firstTimeHelperTextTriggered, data.isSettingsOpen);
        view.layoutDigitalClock(data.clock_option, data.isSettingsOpen);
        view.resize(data.isSettingsOpen, data.tfhours);
        _this.updateThumbnail();
    };
    this.ratioFor = function(clockOption, isSettingsOpen, hasHelpText) {
        /* w / h */
        if(isSettingsOpen) {
            if(clockOption === "analog_digital") {
                return 0.5;
            } else if(clockOption === "digital") {
                return 0.95;
            } else {
                if(hasHelpText) {
                    return 0.5;
                } else {
                    return 0.65;
                }
            }
        } else {
            if(clockOption === "analog_digital") {
                return 0.71;
            } else if(clockOption === "digital") {
                return 2.5;
            } else {
                return 1;
            }
        }
    };
    this.updateThumbnail = function(){
        // add delay to give nb some time to update its internal bitmaps
        setTimeout(function() {
            NB.page.updateThumbnail();
        }, 100);
    };
    //increments the data.date object by 1 second if keep_counting is true
    this.count_time = function() {
        if(_this.getKeepCounting()){
            data.date.setSeconds(data.date.getSeconds() + 1);
            update_time();
        }
    };
    this.hideSecondsToggle = function(){
        if(!data.loaded) {
            data.hide_seconds = !data.hide_seconds;
        }
        view.hideSeconds_View(data.hide_seconds);
        _this.resizeHandler();
        _this.updateArrows();
        model.saveData(data);
    };
    this.tfhoursToggle = function(){
        if(!data.loaded){
            data.tfhours = !data.tfhours;
        }
        view.tfhours_View(data.tfhours);
        _this.resizeHandler();
        _this.updateArrows();
        model.saveData(data);
        update_time();
    };
    this.displayDropdown = function(){
        view.displayDropdown(data.clock_option);
    };
    this.displayOption = function(option){
        data.firstTimeHelperTextTriggered = true;
        $('#firstTimeHelperText').hide();
        _this.switchTo(option, data.isSettingsOpen);
        _this.updateThumbnail();
    };
    this.switchTo = function(option, isSettingsOpen) {
        data.isSettingsOpen = isSettingsOpen;
        data.clock_option = option;
        model.saveData(data);
        if(isSettingsOpen) {
            view.openSettings(data.firstTimeHelperTextTriggered);
        } else {
            view.closeSettings(data.firstTimeHelperTextTriggered, data.clock_option, _this.getKeepCounting());
            data.firstTimeHelperTextTriggered = true;
        }
        var ratio = _this.ratioFor(option, isSettingsOpen, !data.firstTimeHelperTextTriggered);
        var width = $(window).width();
        var host = NB.getHostObject();
        _this.setAnnotationSize(host.width, host.width / ratio);
        view.update_clockView(option);
        view.analogSettings.setShown(true);
        view.digitalSettings.setShown(true);
        _this.resizeHandler();
        setTimeout(_this.resizeHandler, 50);
    };
    //updates the analog and digital views to the current time of the data.date object
    function update_time() {
        var hours = data.date.getHours();
        var minutes = data.date.getMinutes();
        var seconds = data.date.getSeconds();
        var noon = true;
        if (!data.tfhours) {
            if (hours == 0) {
                hours = 12;
                noon = false;
            } else if (hours == 12) {
                noon = true;
            } else if (hours > 12) {
                hours = hours % 13 + 1;
                noon = true;
            } else if (hours < 12) {
                noon = false;
            }
        }
        view.update_time_View(hours, minutes, seconds, noon,_this.getKeepCounting(), data.tfhours);
        model.saveData(data);
    }
    this.playpauseButton = function(){
        if(_this.getKeepCounting()) {
            set();
        } else {
            start();
        }
    };
    //pauses the clock and shows the time input textboxes and increment/decrement buttons for the digital view
    function set() {
        _this.setKeepCounting(false);
        setMode('hour');
        setMode('minute');
        setMode('second');
        view.set_View();
        update_time();
        _this.resizeHandler();
        model.saveData(data);
    }
    //restarts the clock
    function start() {
        setDateFromTyping();
        view.start_View(data.tfhours);
        update_time();
        _this.resizeHandler();
        _this.setKeepCounting(true);
    }
    this.increment = function(h, m, s) {
        data.date.setHours(data.date.getHours() + h);
        data.date.setMinutes(data.date.getMinutes() + m);
        data.date.setSeconds(data.date.getSeconds() + s);
        update_time();
    };
    //alternates the 12h clock between am and pm
    this.toggleNoon = function() {
        if (data.date.getHours() < 12) {
            view.toggleNoon_View(false);
            data.date.setHours(data.date.getHours() + 12);
        } else {
            view.toggleNoon_View(true);
            data.date.setHours(data.date.getHours() - 12);
        }
        update_time();
    };
    //resets data.date object to the current system time
    this.reset = function() {
        data.date = new Date();
        update_time();
        _this.resizeHandler();
    };
    function isNum(event) {
        var keyCode = event.which||event.keyCode;
        if ((keyCode < 48 || keyCode > 57) && keyCode > 31) { //if not a number, and not tab,enter...
            return false;
        }
        return true;
    }
    //function is run whenever a key is pressed
    this.checkTimeInput = function(id,event) {
        var keyCode = event.which || event.keyCode;
        if ((keyCode < 48 || keyCode > 57) && keyCode!=0 && keyCode != 8 && keyCode != 127 && keyCode != 9&& keyCode != 40&& keyCode != 38) { //if not a number or backspace, return
            if (keyCode == 13) {
                start();
            }
            return;
        }
        if(keyCode == 38 || keyCode == 40){
            view.refreshInputVales(data.date.getHours(), data.date.getMinutes(), data.date.getSeconds());
            return;
        }
        //selectedText variable used to fix bug encoutered when holding down 2 number keys at once
        var selectedText = "";
        if (window.getSelection) {
            selectedText = window.getSelection().toString();
        }
        if (id == "hoursinput") {
            if (data.tfhours && document.getElementById(id).value >= 24) {
                document.getElementById(id).value = 23;
            } else if (!data.tfhours && document.getElementById(id).value > 12) {
                document.getElementById(id).value = 12;
            }
        } else {
            if (document.getElementById(id).value >= 60) {
                document.getElementById(id).value = 59;
            }
        }

        setDateFromTyping();

        var selectText = false;

        if (selectedText.length != 0) {
            selectText = true;
        } else if (id == "hoursinput") {
            if (document.getElementById(id).value.length == 2 && (!autoTab || document.getElementById(id).value.charAt(0) == 0)) {
                document.getElementById("minutesinput").select();
                autoTab = true;
                selectText = true;
            } else {
                autoTab = false;
            }
        } else {
            if (!data.hide_seconds && document.getElementById(id).value.length == 2 && id == "minutesinput" && (!autoTab || document.getElementById(id).value.charAt(0) == 0)) {
                document.getElementById("secondsinput").select();
                autoTab = true;
                selectText = true;
            } else {
                autoTab = false;
            }
        }

        update_time();
        _this.updateThumbnail();

        if (selectText) {
            document.getElementById(document.activeElement.id).select();
        }
    };
    //sets the data.date when inputting through keyboard
    function setDateFromTyping() {
        if($("#hoursinput").val() == undefined || $("#minutesinput").val() == undefined || $("#secondsinput").val() == undefined) return;
        var hour;
        if ($("#hoursinput").val().length == 0) {
            hour = 0;
        } else {
            hour = parseInt($("#hoursinput").val());
        }
        if ($("#minutesinput").val().length == 0) {
            $("#minutesinput").val(0);
        }
        if ($("#secondsinput").val().length == 0) {
            $("#secondsinput").val(0);
        }
        if (!data.tfhours && data.date.getHours() >= 12) { //if PM
            if (hour == 12) {
                data.date.setHours(12);
            } else {
                data.date.setHours(hour + 12);
            }
        } else {
            if (!data.tfhours && parseInt($("#hoursinput").val()) == 12) {
                data.date.setHours(0);
            } else {
                data.date.setHours(hour);
            }
        }

        data.date.setMinutes(parseInt($("#minutesinput").val()));
        data.date.setSeconds(parseInt($("#secondsinput").val()));
        _this.updateThumbnail();
    }
    function setMode(string){
        _this.setKeepCounting(false);
        var id="#"+string+"s";
        var input = view.setMode_View(id, data.date, data.tfhours);
        $(input).keyup(function(evt){
            _this.checkTimeInput(id.substring(1,id.length)+"input",evt);
        }).keypress(function(evt){
            return isNum(evt)
        }).focusout(function(){
            _this.focusOut();
        });

    }
    this.focusOut = function() {
        setDateFromTyping();
        update_time();
    };
    this.settingsClickListener = function(){
        var open = !data.isSettingsOpen;
        if(!open) {
            data.firstTimeHelperTextTriggered = true;
            $('#firstTimeHelperText').hide();
        }
        _this.switchTo(data.clock_option, !data.isSettingsOpen);
    };
    this.mouseDownListener =function(evt) {
    };
    this.mouseUpListener = function(evt) {
        if(_this.selectedHand == undefined){
            return;
        }
        data.date = view.updateAngle(event, data.date, _this.selectedHand);
        update_time();
        _this.resizeHandler();
        model.saveData(data);
    };
    this.mouseOverListener = function(evt) {
        if(_this.selectedHand!=undefined){
            data.date = view.updateAngle(event, data.date, _this.selectedHand);
            update_time();
            return true;
        }
    };
    this.onSelectionChange = function(listener, isSelected) {
        view.digitalSettings.setShownAnim(isSelected);
        view.analogSettings.setShownAnim(isSelected);
    };
    this.updateArrows = function() {
        var x = 0;
        if(data.tfhours) {
            x = data.hide_seconds ? 40 : 28;
        } else {
            x = data.hide_seconds ? 30 : 18;
        }
        $('#incButtons, #decButtons').css({
            left: x + '%',
        });
    };
    this.addEvents = function() {
        $(window).resize(function(){
            _this.resizeHandler();
        });
        $("#settings_close").click(function(){
            _this.settingsClickListener();
        });
        view.analogSettings.back.click(function() {
            _this.settingsClickListener();
        });
        view.digitalSettings.back.click(function() {
            _this.settingsClickListener();
        });
        $("#clock").on('mousemove',function(evt){
            var a = _this.mouseOverListener(evt);
        });
        $(".hand").mousedown(function(evt){
            _this.selectedHand = view.get_selectedHand(evt, _this.getDate());
            var isHandSelected = _this.mouseOverListener(evt);
            if(_this.getKeepCounting() == true){
                wasPreviouslyCounting = true;
            } else {
                wasPreviouslyCounting = false;
            }
            if(isHandSelected){
                _this.setKeepCounting(false);
            }
            _this.mouseDownListener(evt);
            $("#clock").on("mouseleave", function(evt) {
                $(".hand").mouseup();
                $(this).off(evt);
            });
        });
        $("#clock").mouseup(function(evt){
            _this.mouseUpListener(evt);
            if (_this.selectedHand !== undefined) {
                if(wasPreviouslyCounting) {
                    _this.setKeepCounting(true);
                } else {
                    _this.setKeepCounting(false);
                }
            }
            _this.selectedHand = undefined;
            _this.resizeHandler();
        });
        $("#clockOptionButton").click(function(){
            if($(".drop-main").is(':visible')) {
                $(".drop-main").hide();
            } else {
                _this.displayDropdown();
            }
        });
        $("#clockOptionButton_analog").click(function(){
            _this.displayOption("analog");
        });
        $("#clockOptionButton_digital").click(function(){
            _this.displayOption("digital");
        });
        $("#clockOptionButton_analog_digital").click(function(){
            _this.displayOption("analog_digital");
        });
        $("#pauseContainer").click(function(){
            _this.playpauseButton();
        });
        $(".timeText").click(function(evt){
            if($("#button_grid").is(":visible")) {
                _this.playpauseButton();
                switch (evt.target.id) {
                    case "hours":
                    case "hours_buffer":
                        $("#hoursinput").focus();
                        break;
                    case "minutes":
                    case "minutes_buffer":
                        $("#minutesinput").focus();
                        break;
                    case "seconds":
                    case "seconds_buffer":
                    default:
                        $("#secondsinput").focus();
                        break;
                }
            }
        });
        $("#resetButton").click(function(){
            _this.reset();
        });
        $("#hideShowButton").click(function(){
            _this.hideSecondsToggle();
        });
        $("#tfhourButton").click(function(){
            _this.tfhoursToggle();
        });
        $("#pauseButton").click(function(){
            _this.playpauseButton()
        });
        $("#ampm").click(function(){
            _this.toggleNoon();
        });
        $("#inchour").click(function(evt) {
            _this.increment(1, 0, 0);
        });
        $("#incminute").click(function(evt) {
            _this.increment(0, 1, 0);
        });
        $("#incsecond").click(function(evt) {
            _this.increment(0, 0, 1);
        });
        $("#dechour").click(function(evt) {
            _this.increment(-1, 0, 0);
        });
        $("#decminute").click(function(evt) {
            _this.increment(0, -1, 0);
        });
        $("#decsecond").click(function(evt) {
            _this.increment(0, 0, -1);
        });
        $('html').mousedown(function(event) { 
            if($('.drop-main').is(":visible")) {
                $('.drop-main').hide()
            }
        });
        $('#clockOptionButton, .drop-option').mousedown(function(event) { 
            event.stopPropagation();
        });
        $(window).focusin(function(event) { 
            view.digitalSettings.setShownAnim(true);
            view.analogSettings.setShownAnim(true);
        });
        $(window).focusout(function(event) {
            if(!_this.selectionListener.isSelected()) {
                view.digitalSettings.setShownAnim(false);
                view.analogSettings.setShownAnim(false);
            }
        });    
    };
    this.init();
}